;
; *.*
;   OCM-EXTRA Pack / Switched I/O ports ($40-$4F) Revision 1 or later
;
; Copyright (c) 2011-2017 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: OCM-GETRESET v1.1 by KdL
;     Date: 2017.09.18
;     Note: OCM-PLD Pack v2.4 or later
;     Coded in TWZ'CA3 w/ TASM80 v3.2ud
; ----------------------------------------
;

            .ORG  $0100

FN1         .EQU  'g'                     ; strings and options
FN2         .EQU  'e'
FN3         .EQU  't'
FN4         .EQU  'r'
FN5         .EQU  'e'
FN6         .EQU  's'
FN7         .EQU  'e'
FN8         .EQU  't'
VER1        .EQU  '1'
VER2        .EQU  '1'
YR1         .EQU  '2'
YR2         .EQU  '0'
YR3         .EQU  '1'
YR4         .EQU  '7'
VERBOSE     .EQU  '/'
QUIET       .EQU  '-'
OPT1        .EQU  '1'
OPT2        .EQU  '2'
OPT3        .EQU  '3'
OPT4        .EQU  '4'
OPTCOLD     .EQU  'c'
OPTWARM     .EQU  'w'
LOCK        .EQU  'l'
UNLOCK      .EQU  'u'
HLP1        .EQU  'h'
HLP2        .EQU  '?'

SCMDLOCK    .EQU  $35
SCMDUNLK    .EQU  SCMDLOCK+1
SWIOADDR    .EQU  $40                     ; switched i/o port
SCMDADDR    .EQU  $41                     ; smart command port
LOCKADDR    .EQU  $43                     ; lock mask port
REVADDR     .EQU  $4f
NULID       .EQU  $80
OCMID       .EQU  $d4                     ; ocm id
SCMDOPT1    .EQU  $fb
SCMDBASE    .EQU  OPT1-SCMDOPT1

RESETLOCK   .EQU  %00100000               ; masks
LOWERCASE   .EQU  %00100000
UP          .EQU  -LOWERCASE
_           .EQU  %10000000               ; obfuscation mask, e.g. 'A'|_,'B'|_
DEOBF       .EQU  %01111111               ; de-obfuscation mask
REVMASK     .EQU  %00011111

NUL         .EQU  $00                     ; ascii table
LF          .EQU  $0a
CLS         .EQU  $0c
CR          .EQU  $0d
EOF         .EQU  $1a
SPC         .EQU  $20

PLENGHT     .EQU  $0080                   ; system calls
CALSLT      .EQU  $001c
CHPUT       .EQU  $00a2
EXPTBL      .EQU  $fcc1
_BDOS       .EQU  $0005
_STROUT     .EQU  $09                     ; string output

ONE         .EQU  $01                     ;   1 byte  for char
HLN         .EQU  $10                     ;  16 bytes for hex line
CLU         .EQU  $0100                   ; 256 bytes for cluster

K1          .EQU  'K'                     ; editable copyleft: 1st three chars
K2          .EQU  'd'
K3          .EQU  'L'
_1          .EQU  K1|_                    ; obfuscated copyleft
_2          .EQU  K2|_
_3          .EQU  K3|_
; ----------------------------------------
startProg:
            jr    mainProg                ; type support, A:\>TYPE FILENAME.COM
typeMsg:
;           .DB   CR,SPC,SPC,SPC,SPC,LF   ; 'jr mainProg' ctrl sequences mask
            .DB   CR,"OCM-"
            .DB   FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   " v",VER1,".",VER2," by "
clearCopyleft:
            .DB   K1,K2,K3," 2011-",YR1,YR2,YR3,YR4,CR
msgLF:
            .DB   LF,EOF
; ----------------------------------------
EXTRA:      .DB   NUL                     ; vars
; ----------------------------------------
mainProg:
            di
            ld    c,LF+ONE                ; init 'call strDisp' to use
            ld    d,OPT3                  ; reg d is 'OPTION'
            ld    e,VERBOSE               ; reg e is 'PREFIX'
copyleftChk:
            ld    hl,clearCopyleft
            ld    a,_1
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
            jr    nz,copyleftNext         ; protection: 'K' is confirmed here
            inc   hl
            ld    a,_2
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
copyleftNext:
            jr    nz,jrnzSPC              ; protection: 'd' is confirmed here
            inc   hl
            ld    a,_3
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
jrnzSPC:
            jr    nz,noParam              ; protection: parameter down!
; ----------------------------------------
startParser:
            ld    hl,PLENGHT
            ld    b,(hl)
            ld    a,b
            cp    NUL
            jr    z,noParam               ; jump if no chars
; ========================================
;           ld    a,(PLENGHT+ONE)         ; it's a bad idea on 'getreset' to
;           cp    SPC                     ; ignore this policy:
;           jr    nz,noParam              ; e.g. A:\>FILENAME/x
; ========================================
preLoop:
            inc   hl
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,getParam             ; get 1st parameter:
            djnz  preLoop                 ; e.g. A:\>FILENAME      -x
noParam:
            jr    ocmidChk                ; no parameters
getParam:
            ld    a,(hl)
            ld    e,a
            djnz  getOption               ; if no option, get invalid parameter
invalidParam:
            ld    hl,errorMsg0
            call  strDisp                 ; 'invalid parameter' without prefix
            jr    nearestExit             ; jump to the nearest exit, 2 bytes
getOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 1st option forced to lowercase
            ld    d,a
            djnz  nextOption              ; if end of PLENGHT, parameter is get
            jr    prefixChk
nextOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 2nd option forced to lowercase
            cp    LOCK
            jr    z,setOpt2               ; is 'l'
            cp    UNLOCK
            jr    nz,postLoop1            ; is other
setOpt2:
            ld    (EXTRA),a               ; is 'u'
            jr    postLoop2
postLoop:
            inc   hl
postLoop1:
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,invalidParam         ; get 2nd parameter as invalid:
postLoop2:
            djnz  postLoop                ; e.g. A:\>FILENAME -x    abcdef
prefixChk:
            ld    a,e
            cp    QUIET                   ; '-'
            jr    z,paramChk
            cp    VERBOSE                 ; '/'
            jr    nz,invalidParam
; ----------------------------------------
paramChk:
            ld    a,d
            cp    OPTCOLD
            jr    z,coldChk               ; jump if 'c'
            cp    OPTWARM
            jr    z,warmChk               ; jump if 'w'
            cp    OPT2
            jr    z,ocmidChk              ; jump if '2'
            cp    OPT4
            jr    z,ocmidChk              ; jump if '4'
            ld    a,(EXTRA)
            cp    b
            jr    nz,invalidParam         ; invalid with '/ll', '-?u', etc.
            ld    a,d
            cp    LOCK
            jr    z,extraOnly             ; jump if 'l'
            cp    UNLOCK
            jr    z,extraOnly             ; jump if 'u'
            cp    HLP1
            jr    z,isHelp                ; jump if 'h'
            cp    HLP2
            jr    nz,invalidOption        ; jump if '?'
isHelp:
            ld    hl,helpMsg              ; help message
nearestExit:
            jr    lastDisp
invalidOption:
            ld    hl,errorMsg0
            call  strDisp
            ld    hl,errorMsg2
            jr    lastDisp                ; 'invalid option'
coldChk:
            ld    d,OPT1
            jr    ocmidChk
warmChk:
            ld    d,OPT3
            jr    ocmidChk
extraOnly:
            ld    (EXTRA),a               ; shift the option to extra
            ld    d,NULID+SCMDBASE        ; $80 is the null smart command
; ----------------------------------------
ocmidChk:
            ld    a,OCMID                 ; check for 1chipMSX ID
            out   (SWIOADDR),a
            in    a,(SWIOADDR)
            cpl
            cp    OCMID
; ----------------------------------------
            jr    nz,softReset            ; 'soft reset' (rem to test)
            in    a,(REVADDR)             ; revision check
            and   REVMASK
            cp    NUL                     ; = revision 0
            jr    z,softReset             ; 'soft reset' (jr nz to test)
; ----------------------------------------
extraChk:
            ld    a,(EXTRA)
            cp    UNLOCK
            jr    nz,lockChk
            ld    a,SCMDUNLK
            jr    setExtra
lockChk:
            cp    LOCK
            jr    nz,outScmd
            ld    a,SCMDLOCK
setExtra:
            out   (SCMDADDR),a            ; lock/unlock hard reset key
outScmd:
            ld    a,d
            cp    NULID+SCMDBASE
            jr    z,quietChk
            sub   SCMDBASE
            out   (SCMDADDR),a            ; set smart command
softReset:
            ld    ix,NUL                  ; unexpected error
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; and soft reset
            ld    hl,unexpMsg             ; 'unexpected error' if CALSLT fails
            jr    lastDisp
quietChk:
            ld    a,e
            cp    QUIET
            jr    z,mainExit              ; quiet exit
; ----------------------------------------
            ld    hl,msg0                 ; 'Hard reset '
            call  strDisp
            ld    b,RESETLOCK             ; locked/unlocked message
            call  strLock
; ----------------------------------------
lastDisp:
            dec   c                       ; use 'jr lastDisp' or 'jp lastDisp'
strDisp:
            inc   c                       ; use 'call strDisp'
charLoop:
            ld    a,(hl)                  ; load 1st char, ascii < 127 only
            and   DEOBF                   ; de-obfuscation
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; NUL and EOF are not valid as 1st
            inc   hl
            xor   a
            cp    (hl)
            jr    nz,nextChar             ; NUL,n = SPC(n)
            inc   hl
            ld    b,(hl)                  ; b = n
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
;           and   DEOBF                   ; de-obfuscation (optional)
spcLoop:
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; print SPC until b = 0
            djnz  spcLoop
            inc   hl
nextChar:
            ld    a,EOF
            cp    (hl)
            jr    nz,charLoop             ; EOF = end of string
            inc   hl                      ; point to the next string at exit
            dec   c
            ld    a,LF
            cp    c                       ; c = LF is 'lastDisp'
            ret   nz                      ; exit of 'strDisp'
            ld    de,strLFCR              ; print LF + CR after 'lastDisp'
            ld    c,_STROUT               ; 'OPTION' and 'PREFIX' will be lost
            call  _BDOS
mainExit:
            xor   a
            ld    (PLENGHT),a             ; reset PLENGHT for the next command
            ei
            ret
strLFCR:
            .DB   LF,CR,'$'
; ----------------------------------------
strLock:
            in    a,(LOCKADDR)
            and   b                       ; b is required as lock mask here
            cp    NUL
            jr    z,msgUnlock
            ld    hl,lockMsg
            ret
msgUnlock:
            ld    hl,unlockMsg
            ret
; ----------------------------------------
helpMsg:                                  ; texts comply with 37 columns
            .DB   LF|_
            .DB   CLS|_,'S'|_,'y'|_,'n'|_,'t'|_,'a'|_,'x'|_,':'|_,' '|_
            .DB   FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   '['|_,'{'|_,VERBOSE|_,'|'|_,QUIET|_,'}'|_
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_
            .DB   '['|_,'l'|_,'|'|_,'u'|_,']'|_,']'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'N'|_,'o'|_,' '|_
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,':'|_
            .DB   NUL,4,'W'|_,'a'|_,'r'|_,'m'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'P'|_,'r'|_,'e'|_,'f'|_,'i'|_,'x'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,4,VERBOSE|_,NUL,3
            .DB   'V'|_,'e'|_,'r'|_,'b'|_,'o'|_,'s'|_,'e'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_,CR|_,NUL,13,QUIET|_,NUL,3
            .DB   'Q'|_,'u'|_,'i'|_,'e'|_,'t'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'O'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,'s'|_,':'|_
            .DB   NUL,5,OPTCOLD|_,NUL,3,'C'|_,'o'|_,'l'|_,'d'|_,' '|_,
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,LF|_,CR|_,NUL,13,OPTWARM|_
            .DB   NUL,3,'W'|_,'a'|_,'r'|_,'m'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,LF|_,CR|_,NUL,13,OPT2|_
            .DB   NUL,3,'W'|_,'a'|_,'r'|_,'m'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,' '|_,'2'|_,'M'|_,'B'|_
            .DB   LF|_,CR|_,NUL,13,OPT4|_,NUL,3,'W'|_,'a'|_,'r'|_,'m'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,' '|_,'4'|_,'M'|_,'B'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'E'|_,'x'|_,'t'|_,'r'|_,'a'|_,'s'|_,':'|_,NUL,6
            .DB   LOCK,NUL,3,'H'|_,'a'|_,'r'|_,'d'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,' '|_
            .DB   'l'|_,'o'|_,'c'|_,'k'|_,'e'|_,'d'|_,LF|_,CR|_,NUL,13,UNLOCK
            .DB   NUL,3,'H'|_,'a'|_,'r'|_,'d'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,' '|_
            .DB   'u'|_,'n'|_,'l'|_,'o'|_,'c'|_,'k'|_,'e'|_,'d'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'E'|_,'x'|_,'a'|_,'m'|_,'p'|_,'l'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,8,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_
            .DB   LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   VERBOSE|_,UNLOCK|_,LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   QUIET|_,OPT4|_,LOCK|_

            .DB   EOF
msg0:
            .DB   'H'|_,'a'|_,'r'|_,'d'|_,' '|_
            .DB   'r'|_,'e'|_,'s'|_,'e'|_,'t'|_,' '|_,EOF
unlockMsg:
            .DB   'u'|_,'n'|_
lockMsg:
            .DB   'l'|_,'o'|_,'c'|_,'k'|_,'e'|_,'d'|_,EOF
errorMsg0:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'I'|_,'n'|_,'v'|_,'a'|_,'l'|_,'i'|_,'d'|_,' '|_,EOF
errorMsg1:
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,EOF
errorMsg2:
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,EOF
unexpMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'U'|_,'n'|_,'e'|_,'x'|_,'p'|_,'e'|_,'c'|_,'t'|_,'e'|_,'d'|_
            .DB   ' '|_,'e'|_,'r'|_,'r'|_,'o'|_,'r'|_,EOF
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
            .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;           .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
            .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   ".COM ]"

            .END
