;
; *.*
;   OCM-EXTRA Pack / Switched I/O ports ($40-$4F) Revision 1 or later
;
; Copyright (c) 2011-2017 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: OCM-SETSMART v1.2 by KdL
;     Date: 2017.09.18
;     Note: OCM-PLD Pack v2.4 or later
;     Coded in TWZ'CA3 w/ TASM80 v3.2ud
; ----------------------------------------
;

            .ORG  $0100

FN1         .EQU  's'                     ; strings and options
FN2         .EQU  'e'
FN3         .EQU  't'
FN4         .EQU  's'
FN5         .EQU  'm'
FN6         .EQU  'a'
FN7         .EQU  'r'
FN8         .EQU  't'
VER1        .EQU  '1'
VER2        .EQU  '2'
YR1         .EQU  '2'
YR2         .EQU  '0'
YR3         .EQU  '1'
YR4         .EQU  '7'
VERBOSE     .EQU  '/'
QUIET       .EQU  '-'

NINE        .EQU  $09
FIFTEEN     .EQU  $0f
SWIOADDR    .EQU  $40                     ; switched i/o port
SCMDADDR    .EQU  $41                     ; smart command port
REVADDR     .EQU  $4f
NULID       .EQU  $80
OCMID       .EQU  $d4                     ; ocm id
ERR         .EQU  $ff

LOWERCASE   .EQU  %00100000               ; masks
UP          .EQU  -LOWERCASE
_           .EQU  %10000000               ; obfuscation mask, e.g. 'A'|_,'B'|_
DEOBF       .EQU  %01111111               ; de-obfuscation mask
REVMASK     .EQU  %00011111
LOWEROBF    .EQU  LOWERCASE|_

OPT0        .EQU  '0'|_
OPT1        .EQU  '1'|_
OPT2        .EQU  '2'|_
OPT3        .EQU  '3'|_
OPT4        .EQU  '4'|_
OPT5        .EQU  '5'|_
OPT6        .EQU  '6'|_
OPT7        .EQU  '7'|_
OPT8        .EQU  '8'|_
OPT9        .EQU  '9'|_
OPTA        .EQU  'a'|_
OPTB        .EQU  'b'|_
OPTC        .EQU  'c'|_
OPTD        .EQU  'd'|_
OPTE        .EQU  'e'|_
OPTF        .EQU  'f'|_

NUL         .EQU  $00                     ; ascii table
LF          .EQU  $0a
CLS         .EQU  $0c
CR          .EQU  $0d
EOF         .EQU  $1a
SPC         .EQU  $20

PLENGHT     .EQU  $0080                   ; system calls
CALSLT      .EQU  $001c
CHPUT       .EQU  $00a2
EXPTBL      .EQU  $fcc1
_BDOS       .EQU  $0005
_STROUT     .EQU  $09                     ; string output

ONE         .EQU  $01                     ;   1 byte  for char
HLN         .EQU  $10                     ;  16 bytes for hex line
CLU         .EQU  $0100                   ; 256 bytes for cluster

K1          .EQU  'K'                     ; editable copyleft: 1st three chars
K2          .EQU  'd'
K3          .EQU  'L'
_1          .EQU  K1|_                    ; obfuscated copyleft
_2          .EQU  K2|_
_3          .EQU  K3|_
; ----------------------------------------
startProg:
            jr    mainProg                ; type support, A:\>TYPE FILENAME.COM
typeMsg:
;           .DB   CR,SPC,SPC,SPC,SPC,LF   ; 'jr mainProg' ctrl sequences mask
            .DB   CR,"OCM-"
            .DB   FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   " v",VER1,".",VER2," by "
clearCopyleft:
            .DB   K1,K2,K3," 2011-",YR1,YR2,YR3,YR4,CR
msgLF:
            .DB   LF,EOF
; ----------------------------------------
mainProg:
            di
            ld    c,LF+ONE                ; init 'call strDisp' to use
copyleftChk:
            ld    hl,clearCopyleft
            ld    a,_1
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
            jr    nz,copyleftNext         ; protection: 'K' is confirmed here
            inc   hl
            ld    a,_2
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
copyleftNext:
            jr    nz,jrnzSPC              ; protection: 'd' is confirmed here
            inc   hl
            ld    a,_3
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
jrnzSPC:
            jr    nz,noParam              ; protection: parameter down!
; ----------------------------------------
startParser:
            ld    hl,PLENGHT
            ld    b,(hl)
            ld    a,b
            cp    NUL
            jr    z,noParam               ; jump if no chars
            ld    a,(PLENGHT+ONE)
            cp    SPC                     ; ignore this policy:
            jr    nz,noParam              ; e.g. A:\>FILENAME/x
preLoop:
            inc   hl
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,getParam             ; get 1st parameter:
            djnz  preLoop                 ; e.g. A:\>FILENAME      -x
noParam:
            ld    e,NUL
            ld    hl,helpMsg              ; help message
            jr    ocmidChk
getParam:
            ld    a,(hl)
            ld    e,a
            cp    QUIET                   ; '-'
            jr    z,ocmidChk
            cp    VERBOSE                 ; '/'
            jr    z,ocmidChk
invalidParam:
            ld    hl,errorMsg0
            call  strDisp                 ; 'invalid parameter' without prefix
            jr    lastDisp                ; jump to the nearest exit, 2 bytes
invalidChar:
            ld    a,(hl)
            push  af
            ld    hl,errorMsg0
            call  strDisp
            ld    hl,errorMsg2
            call  strDisp                 ; 'invalid char ['
            pop   af
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT
            ld    hl,errorMsg3
            jr    lastDisp                ; ']'
invalidCode:
            ld    hl,errorMsg0
            call  strDisp
            ld    hl,errorMsg4            ; 'invalid code'
            jr    lastDisp
; ----------------------------------------
ocmidChk:
            ld    a,OCMID                 ; check for 1chipMSX ID
            out   (SWIOADDR),a
            in    a,(SWIOADDR)
            cpl
            cp    OCMID
; ----------------------------------------
            jr    nz,isUnsupp             ; 'unsupported command' (rem to test)
            in    a,(REVADDR)             ; revision check
            and   REVMASK
            cp    NUL                     ; = revision 0
            jr    z,isUnsupp              ; 'unsupported command' (jr nz to test)
; ----------------------------------------
            ld    a,e
            cp    NUL
            jr    z,lastDisp              ; help message
            dec   b
            ld    a,b
            cp    NUL
            jr    z,invalidParam
parseLoop:
            inc   hl
            ld    a,(hl)
            or    LOWEROBF
            dec   c
            call  getChar                 ; 1st char
            inc   c
            cp    ERR
            jr    z,invalidChar
            ld    d,a
            dec   b
            ld    a,b
            cp    NUL
            jr    z,invalidCode
            inc   hl
            ld    a,(hl)
            or    LOWEROBF
            call  getChar                 ; 2nd char
            cp    ERR
            jr    z,invalidChar
            or    d
outScmd:
            out   (SCMDADDR),a            ; set smart command
            djnz  parseLoop
quietChk:
            ld    a,e
            cp    QUIET
            jr    z,quietExit             ; quiet exit
; ----------------------------------------
getStatus:
            ld    hl,msgDone              ; 'Done!'
            jr    lastDisp                ; display last string and exit
isUnsupp:
            ld    a,e
            cp    QUIET
quietExit:
            jr    z,mainExit              ; quiet exit (useful for batch)
            ld    hl,unsuppMsg            ; 'unsupported command' if '/' prefix
; ----------------------------------------
lastDisp:
            dec   c                       ; use 'jr lastDisp' or 'jp lastDisp'
strDisp:
            inc   c                       ; use 'call strDisp'
charLoop:
            ld    a,(hl)                  ; load 1st char, ascii < 127 only
            and   DEOBF                   ; de-obfuscation
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; NUL and EOF are not valid as 1st
            inc   hl
            xor   a
            cp    (hl)
            jr    nz,nextChar             ; NUL,n = SPC(n)
            inc   hl
            ld    b,(hl)                  ; b = n
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
;           and   DEOBF                   ; de-obfuscation (optional)
spcLoop:
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; print SPC until b = 0
            djnz  spcLoop
            inc   hl
nextChar:
            ld    a,EOF
            cp    (hl)
            jr    nz,charLoop             ; EOF = end of string
            inc   hl                      ; point to the next string at exit
            dec   c
            ld    a,LF
            cp    c                       ; c = LF is 'lastDisp'
            ret   nz                      ; exit of 'strDisp'
            ld    de,strLFCR              ; print LF + CR after 'lastDisp'
            ld    c,_STROUT               ; 'OPTION' and 'PREFIX' will be lost
            call  _BDOS
mainExit:
            xor   a
            ld    (PLENGHT),a             ; reset PLENGHT for the next command
            ei
            ret
strLFCR:
            .DB   LF,CR,'$'
; ----------------------------------------
getChar:
            cp    OPT0                    ; '0'
            jr    z,to0
            cp    OPT1                    ; '1'
            jr    z,to0
            cp    OPT2                    ; '2'
            jr    z,to0
            cp    OPT3                    ; '3'
            jr    z,to0
            cp    OPT4                    ; '4'
            jr    z,to0
            cp    OPT5                    ; '5'
            jr    z,to0
            cp    OPT6                    ; '6'
            jr    z,to0
            cp    OPT7                    ; '7'
            jr    z,to0
            cp    OPT8                    ; '8'
            jr    z,to0
            cp    OPT9                    ; '9'
            jr    z,to0
            cp    OPTA                    ; 'a'
            jr    z,toA
            cp    OPTB                    ; 'b'
            jr    z,toA
            cp    OPTC                    ; 'c'
            jr    z,toA
            cp    OPTD                    ; 'd'
            jr    z,toA
            cp    OPTE                    ; 'e'
            jr    z,toA
            cp    OPTF                    ; 'f'
            jr    z,toA
            ld    a,ERR                   ; $ff with other chars
            ret
toA:
            add   a,NINE
to0:
            and   FIFTEEN
            ld    (startProg),a
            ld    a,LF
            cp    c
            ld    a,(startProg)
            ret   nz                      ; 2nd char exit here
            rlca
            rlca
            rlca
            rlca
            ret                           ; 1st char exit here
; ----------------------------------------
helpMsg:                                  ; texts comply with 37 columns
            .DB   LF|_
            .DB   CLS|_,'S'|_,'y'|_,'n'|_,'t'|_,'a'|_,'x'|_,':'|_,' '|_
            .DB   FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   '['|_,'{'|_,VERBOSE|_,'|'|_,QUIET|_,'}'|_
            .DB   'c'|_,'o'|_,'d'|_,'e'|_,'('|_,'s'|_,')'|_,']'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'N'|_,'o'|_,' '|_
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,':'|_
            .DB   NUL,4,'S'|_,'h'|_,'o'|_,'w'|_,'s'|_,' '|_
            .DB   't'|_,'h'|_,'i'|_,'s'|_,' '|_,'h'|_,'e'|_,'l'|_,'p'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'P'|_,'r'|_,'e'|_,'f'|_,'i'|_,'x'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,4,VERBOSE|_,NUL,3
            .DB   'V'|_,'e'|_,'r'|_,'b'|_,'o'|_,'s'|_,'e'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_,CR|_,NUL,13,QUIET|_,NUL,3
            .DB   'Q'|_,'u'|_,'i'|_,'e'|_,'t'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'C'|_,'o'|_,'d'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,6,'0'|_,'0'|_,NUL,3,'F'|_,'i'|_,'r'|_,'s'|_,'t'|_,' '|_
            .DB   'c'|_,'o'|_,'m'|_,'m'|_,'a'|_,'n'|_,'d'|_,LF|_,CR|_
            .DB   NUL,12,'|'|_,'|'|_,NUL,5,'|'|_,NUL,5,'|'|_
            .DB   NUL,3,LF|_,CR|_,NUL,12,'f'|_,'f'|_
            .DB   NUL,3,'L'|_,'a'|_,'s'|_,'t'|_,' '|_
            .DB   'c'|_,'o'|_,'m'|_,'m'|_,'a'|_,'n'|_,'d'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'E'|_,'x'|_,'a'|_,'m'|_,'p'|_,'l'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,8,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   VERBOSE|_,'1'|_,'a'|_,LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   QUIET|_,'1'|_,'a'|_,'2'|_,'b'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'H'|_,'E'|_,'L'|_,'P'|_,' '|_,FN1+UP|_,FN2+UP|_,FN3+UP|_
            .DB   FN4+UP|_,FN5+UP|_,FN6+UP|_,FN7+UP|_,FN8+UP|_,NUL,4,
            .DB   'L'|_,'i'|_,'s'|_,'t'|_,'s'|_,' '|_,'t'|_,'h'|_,'e'|_,' '|_
            .DB   'c'|_,'o'|_,'d'|_,'e'|_,'s'|_

            .DB   EOF
msgDone:
            .DB   'D'|_,'o'|_,'n'|_,'e'|_,'!'|_,EOF
errorMsg0:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'I'|_,'n'|_,'v'|_,'a'|_,'l'|_,'i'|_,'d'|_,' '|_,EOF
errorMsg1:
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,EOF
errorMsg2:
            .DB   'c'|_,'h'|_,'a'|_,'r'|_,' '|_,'['|_,EOF
errorMsg3:
            .DB   ']'|_,EOF
errorMsg4:
            .DB   'c'|_,'o'|_,'d'|_,'e'|_,EOF
unsuppMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'U'|_,'n'|_,'s'|_,'u'|_,'p'|_,'p'|_,'o'|_,'r'|_,'t'|_,'e'|_
            .DB   'd'|_,' '|_,'c'|_,'o'|_,'m'|_,'m'|_,'a'|_,'n'|_,'d'|_,EOF
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
            .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;           .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
            .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   ".COM ]"

            .END
