;
; *.*
;   OCM-EXTRA Pack / Switched I/O ports ($40-$4F) Revision 7 or later
;
; Copyright (c) 2018-2019 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: OCM-VGARATIO v1.3 by KdL
;     Date: 2019.04.15
;     Note: OCM-PLD Pack v3.6 or later
;     Coded in TWZ'CA3 w/ TASM80 v3.2ud
; ----------------------------------------
;

            .ORG  $0100

FN1         .EQU  'v'                     ; strings and options
FN2         .EQU  'g'
FN3         .EQU  'a'
FN4         .EQU  'r'
FN5         .EQU  'a'
FN6         .EQU  't'
FN7         .EQU  'i'
FN8         .EQU  'o'
VER1        .EQU  '1'
VER2        .EQU  '3'
YR1         .EQU  '2'
YR2         .EQU  '0'
YR3         .EQU  '1'
YR4         .EQU  '9'

SCMD358M    .EQU  $03
SCMD806M    .EQU  $0a
NTSCONLY    .EQU  $d0
AUTOMODE    .EQU  $d1
PIXRATIO    .EQU  $7f
SWIOADDR    .EQU  $40                     ; switched i/o port
SCMDADDR    .EQU  $41                     ; smart command port
VDIPADDR    .EQU  $42                     ; virtual dip-sw port
PANASTAT    .EQU  $48
STATADDR    .EQU  $4a
REVADDR     .EQU  $4f
OCMID       .EQU  $d4                     ; ocm id

TURBOMASK   .EQU  %00000001               ; masks
VGAMASK     .EQU  %00000010
RATIOMASK   .EQU  %00001110
LOWERCASE   .EQU  %00100000
UP          .EQU  -LOWERCASE
_           .EQU  %10000000               ; obfuscation mask, e.g. 'A'|_,'B'|_
DEOBF       .EQU  %01111111               ; de-obfuscation mask
REVMASK     .EQU  %00011111

NUL         .EQU  $00                     ; ascii table
BEL         .EQU  $07
LF          .EQU  $0a
CLS         .EQU  $0c
CR          .EQU  $0d
EOF         .EQU  $1a
SPC         .EQU  $20
QUOTE       .EQU  $22

PLENGHT     .EQU  $0080                   ; system calls
CALSLT      .EQU  $001c
INITXT      .EQU  $006c
INIT32      .EQU  $006f
CHPUT       .EQU  $00a2
LINL40      .EQU  $f3ae
LINL32      .EQU  $f3af
OLDMOD      .EQU  $fcb0
EXPTBL      .EQU  $fcc1
_BDOS       .EQU  $0005
_INNOE      .EQU  $08                     ; console input without echo
_STROUT     .EQU  $09                     ; string output

ONE         .EQU  $01                     ;   1 byte  for char
HLN         .EQU  $10                     ;  16 bytes for hex line
CLU         .EQU  $0100                   ; 256 bytes for cluster

K1          .EQU  'K'                     ; editable copyleft: 1st three chars
K2          .EQU  'd'
K3          .EQU  'L'
_1          .EQU  K1|_                    ; obfuscated copyleft
_2          .EQU  K2|_
_3          .EQU  K3|_
; ----------------------------------------
startProg:
            jr    mainProg                ; type support, A:\>TYPE FILENAME.COM
typeMsg:
;           .DB   CR,SPC,SPC,SPC,SPC,LF   ; 'jr mainProg' ctrl sequences mask
            .DB   CR,"OCM-"
            .DB   FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   " v",VER1,".",VER2," by "
clearCopyleft:
            .DB   K1,K2,K3," 2018-",YR1,YR2,YR3,YR4,CR
msgLF:
            .DB   LF,EOF
; ----------------------------------------
mainProg:
            di
            ld    c,LF+ONE                ; init 'call strDisp' to use
copyleftChk:
            ld    hl,clearCopyleft
            ld    a,_1
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
            jr    nz,copyleftNext         ; protection: 'K' is confirmed here
            inc   hl
            ld    a,_2
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
copyleftNext:
            jr    nz,jrnzSPC              ; protection: 'd' is confirmed here
            inc   hl
            ld    a,_3
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
jrnzSPC:
            jr    nz,noParam              ; protection: parameter down!
; ----------------------------------------
noParam:
            ld    a,OCMID                 ; check for 1chipMSX ID
            out   (SWIOADDR),a
            in    a,(SWIOADDR)
            cpl
            cp    OCMID
; ----------------------------------------
            jp    nz,isUnsupp             ; 'unsupported command' (rem to test)
            in    a,(REVADDR)             ; revision check
            and   REVMASK
            cp    NUL                     ; = revision 0
            jp    z,isUnsupp              ; 'unsupported command' (jr nz to test)
; ----------------------------------------
            ld    b,6                     ; = revision 6, 5, 4, 3, 2, 1
testUnsupp:
            cp    b
            jp    z,isUnsupp              ; 'unsupported command' (jr nz to test)
            djnz  testUnsupp
; ----------------------------------------
            in    a,(VDIPADDR)            ; vga check
            and   VGAMASK
            sub   VGAMASK
            jp    nz,isWrong
; ----------------------------------------
            ld    de,$1100                ; save kanji mode
            call  $ffca
            ld    (startProg),a
            ld    a,(OLDMOD)              ; save current text mode
            ld    (startProg+1),a
            ld    a,(LINL40)              ; save width of text mode 40 or 80
            ld    (startProg+2),a
            ld    a,(LINL32)              ; save width of text mode 32
            ld    (startProg+3),a
; ----------------------------------------
            in    a,(VDIPADDR)            ; save standard speed 3.58MHz
            and   TURBOMASK
            ld    (msgLF),a
            in    a,(PANASTAT)            ; save turbo pana 5.37MHz
            and   TURBOMASK
            ld    (msgLF+1),a
            ld    a,SCMD806M              ; set custom speed 8.06MHz
            out   (SCMDADDR),a
; ----------------------------------------
            xor   a                       ; call ank
            ld    de,$1101
            call  $ffca
            ld    a,80                    ; text mode 80
            ld    (LINL40),a
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            ld    ix,INITXT
            call  CALSLT
setWidth:
            ld    a,PIXRATIO              ; set 'Pixel Ratio 1:1 for LED Display'
            out   (SCMDADDR),a
            in    a,(STATADDR)
            and   RATIOMASK
            cp    4*2                     ; = 640 pixels
            jr    nz,setWidth
            ld    a,NTSCONLY              ; set 'NTSC only'
            out   (SCMDADDR),a
            ei
; ----------------------------------------
            ld    hl,msgWelcome           ; ' Welcome to the screen setup'
            call  strDisp
            ld    a,'0'|_                 ; ' 000000',' 111111',' 222222'
            call  print78x3
            ld    a,'A'|_                 ; ' AAAAAA',' BBBBBB',' CCCCCC'
            call  print78x3
            ld    a,'x'|_                 ; ' xxxxxx',' yyyyyy',' zzzzzz'
            call  print78x3
            ld    hl,msgNow               ; ' NOW YOU CAN PERFORM THE AUTO'
            call  strDisp
            ld    c,_INNOE                ; pause
            call  _BDOS
; ----------------------------------------
            di
            ld    a,PIXRATIO              ; set 'Pixel Ratio 1:1 for LED Display'
            out   (SCMDADDR),a
            out   (SCMDADDR),a
            out   (SCMDADDR),a
            out   (SCMDADDR),a            ; = 'Off'
            ld    a,AUTOMODE              ; set 'NTSC/PAL Auto Mode'
            out   (SCMDADDR),a
; ----------------------------------------
            ld    a,(msgLF+1)
            cp    NUL
            jr    nz,setPana              ; restore turbo pana 5.37MHz
            ld    a,(msgLF)
            cp    NUL
            jr    nz,setScreen
            ld    a,SCMD358M              ; restore standard speed 3.58MHz
            jr    setSpeed
setPana:
            ld    a,SCMDADDR
setSpeed:
            out   (SCMDADDR),a
; ----------------------------------------
setScreen:
            ld    a,(startProg)           ; reload kanji mode
            cp    0
            jr    nz,isKanji
            ld    a,(startProg+1)         ; reload previous text mode
            cp    0
            jr    nz,isMode32
            ld    a,(startProg+2)         ; restore text mode 40 or 80
            ld    (LINL40),a
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            ld    ix,INITXT
            call  CALSLT
exitMsg:
            ei
            ld    hl,msgDone              ; 'Done!'
            jr    lastDisp
; ----------------------------------------
print78x3:
            ld    (jrnzSPC),a             ; '0','A','x'
            ld    hl,msg78ch
            call  strDisp
            ld    a,(jrnzSPC)
            inc   a
            ld    (jrnzSPC),a             ; '1','B','y'
            ld    hl,msg78ch
            call  strDisp
            ld    a,(jrnzSPC)
            inc   a
            ld    (jrnzSPC),a             ; '2','C','z'
            ld    hl,msg78ch
            jr    strDisp
; ----------------------------------------
isKanji:
            ld    de,$1101                ; restore kanji mode
            call  $ffca
            jr    exitMsg
isMode32:
            ld    a,(startProg+3)         ; restore text mode 32
            ld    (LINL32),a
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            ld    ix,INIT32
            call  CALSLT
            jr    exitMsg
isWrong:
            ld    hl,wrongMsg             ; 'wrong video output'
            jr    lastDisp
isUnsupp:
            ld    hl,unsuppMsg            ; 'unsupported command'
; ----------------------------------------
lastDisp:
            dec   c                       ; use 'jr lastDisp' or 'jp lastDisp'
strDisp:
            inc   c                       ; use 'call strDisp'
charLoop:
            ld    a,(hl)                  ; load 1st char, ascii < 127 only
            and   DEOBF                   ; de-obfuscation
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; NUL and EOF are not valid as 1st
            inc   hl
            xor   a
            cp    (hl)
            jr    nz,nextChar             ; NUL,n = SPC(n)
            inc   hl
            ld    b,(hl)                  ; b = n
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            and   DEOBF                   ; de-obfuscation (optional)
spcLoop:
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; print SPC until b = 0
            djnz  spcLoop
            inc   hl
nextChar:
            ld    a,EOF
            cp    (hl)
            jr    nz,charLoop             ; EOF = end of string
            inc   hl                      ; point to the next string at exit
            dec   c
            ld    a,LF
            cp    c                       ; c = LF is 'lastDisp'
            ret   nz                      ; exit of 'strDisp'
            ld    de,strLFCR              ; print LF + CR after 'lastDisp'
            ld    c,_STROUT               ; 'OPTION' and 'PREFIX' will be lost
            call  _BDOS
mainExit:
            xor   a
            ld    (PLENGHT),a             ; reset PLENGHT for the next command
            ei
            ret
strLFCR:
            .DB   LF,CR,'$'
; ----------------------------------------
msgWelcome:                               ; texts comply with 80 columns
            .DB   CLS|_,LF|_
            .DB   ' '|_,'W'|_,'e'|_,'l'|_,'c'|_,'o'|_,'m'|_,'e'|_,' '|_,'t'|_,'o'|_,' '|_,'t'|_,'h'|_,'e'|_
            .DB   ' '|_,'s'|_,'c'|_,'r'|_,'e'|_,'e'|_,'n'|_,' '|_,'s'|_,'e'|_,'t'|_,'u'|_,'p'|_,' '|_,'o'|_,'f'|_
            .DB   ' '|_,QUOTE|_,'P'|_,'i'|_,'x'|_,'e'|_,'l'|_,' '|_,'R'|_,'a'|_,'t'|_,'i'|_,'o'|_,' '|_,'1'|_,':'|_,'1'|_
            .DB   ' '|_,'f'|_,'o'|_,'r'|_,' '|_,'L'|_,'E'|_,'D'|_,' '|_,'D'|_,'i'|_,'s'|_,'p'|_,'l'|_,'a'|_,'y'|_,QUOTE|_
            .DB   ' '|_,'('|_,'6'|_,'0'|_,'H'|_,'z'|_,' '|_,'o'|_,'n'|_,'l'|_,'y'|_,')'|_,'.'|_
            .DB   LF|_,CR|_
            .DB   ' '|_,'A'|_,' '|_,'s'|_,'p'|_,'e'|_,'c'|_,'i'|_,'a'|_,'l'|_,' '|_,'t'|_,'r'|_,'i'|_,'c'|_,'k'|_
            .DB   ' '|_,'d'|_,'e'|_,'s'|_,'i'|_,'g'|_,'n'|_,'e'|_,'d'|_,' '|_,'f'|_,'o'|_,'r'|_
            .DB   ' '|_,'1'|_,'c'|_,'h'|_,'i'|_,'p'|_,'M'|_,'S'|_,'X'|_,' '|_,'a'|_,'n'|_,'d'|_
            .DB   ' '|_,'Z'|_,'e'|_,'m'|_,'m'|_,'i'|_,'x'|_,' '|_,'N'|_,'e'|_,'o'|_,' '|_,'b'|_,'y'|_,' '|_,'K'|_,'d'|_,'L'|_,'.'|_
            .DB   LF|_,CR|_,LF|_

            .DB   ' '|_,'T'|_,'h'|_,'i'|_,'s'|_,' '|_,'s'|_,'c'|_,'r'|_,'e'|_,'e'|_,'n'|_,' '|_,'a'|_,'l'|_,'l'|_,'o'|_,'w'|_,'s'|_
            .DB   ' '|_,'y'|_,'o'|_,'u'|_,' '|_,'t'|_,'o'|_,' '|_,'c'|_,'a'|_,'l'|_,'i'|_,'b'|_,'r'|_,'a'|_,'t'|_,'e'|_,' '|_,'a'|_,'n'|_
            .DB   ' '|_,'L'|_,'E'|_,'D'|_,' '|_,'d'|_,'i'|_,'s'|_,'p'|_,'l'|_,'a'|_,'y'|_,' '|_,'b'|_,'y'|_
            .DB   ' '|_,'t'|_,'e'|_,'m'|_,'p'|_,'o'|_,'r'|_,'a'|_,'r'|_,'i'|_,'l'|_,'y'|_
            .DB   ' '|_,'a'|_,'c'|_,'t'|_,'i'|_,'v'|_,'a'|_,'t'|_,'i'|_,'n'|_,'g'|_
            .DB   LF|_,CR|_
            .DB   ' '|_,'a'|_,'n'|_,' '|_,'e'|_,'n'|_,'l'|_,'a'|_,'r'|_,'g'|_,'e'|_,'d'|_,' '|_,'i'|_,'m'|_,'a'|_,'g'|_,'e'|_
            .DB   ' '|_,'o'|_,'f'|_,' '|_,'6'|_,'4'|_,'0'|_,' '|_,'p'|_,'i'|_,'x'|_,'e'|_,'l'|_,'s'|_,'.'|_
            .DB   ' '|_,'B'|_,'y'|_,' '|_,'e'|_,'x'|_,'i'|_,'t'|_,'i'|_,'n'|_,'g'|_,' '|_,'a'|_,'t'|_,' '|_,'t'|_,'h'|_,'e'|_
            .DB   ' '|_,'p'|_,'r'|_,'o'|_,'m'|_,'p'|_,'t'|_,' '|_,'y'|_,'o'|_,'u'|_,' '|_,'c'|_,'a'|_,'n'|_,' '|_,'t'|_,'r'|_,'y'|_
            .DB   ' '|_,'a'|_,'n'|_,'o'|_,'t'|_,'h'|_,'e'|_,'r'|_
            .DB   LF|_,CR|_
            .DB   ' '|_,'w'|_,'i'|_,'d'|_,'t'|_,'h'|_,' '|_,'b'|_,'e'|_,'t'|_,'w'|_,'e'|_,'e'|_,'n'|_,' '|_,'6'|_,'4'|_,'3'|_
            .DB   ' '|_,'a'|_,'n'|_,'d'|_,' '|_,'6'|_,'3'|_,'7'|_,' '|_,'p'|_,'i'|_,'x'|_,'e'|_,'l'|_,'s'|_,' '|_,'b'|_,'y'|_
            .DB   ' '|_,'e'|_,'x'|_,'e'|_,'c'|_,'u'|_,'t'|_,'i'|_,'n'|_,'g'|_,' '|_,'S'|_,'E'|_,'T'|_,'S'|_,'M'|_,'A'|_,'R'|_,'T'|_
            .DB   ' '|_,'-'|_,'7'|_,'F'|_,' '|_,'s'|_,'t'|_,'e'|_,'p'|_,' '|_,'b'|_,'y'|_,' '|_,'s'|_,'t'|_,'e'|_,'p'|_,'.'|_
            .DB   ' '|_,'T'|_,'o'|_,' '|_,'b'|_,'e'|_
            .DB   LF|_,CR|_
            .DB   ' '|_,'s'|_,'u'|_,'c'|_,'c'|_,'e'|_,'s'|_,'s'|_,'f'|_,'u'|_,'l'|_,' '|_,'w'|_,'i'|_,'t'|_,'h'|_
            .DB   ' '|_,'t'|_,'h'|_,'e'|_,' '|_,'c'|_,'a'|_,'l'|_,'i'|_,'b'|_,'r'|_,'a'|_,'t'|_,'i'|_,'o'|_,'n'|_
            .DB   ' '|_,'y'|_,'o'|_,'u'|_,' '|_,'c'|_,'a'|_,'n'|_,' '|_,'a'|_,'l'|_,'s'|_,'o'|_,' '|_,'t'|_,'r'|_,'y'|_
            .DB   ' '|_,'v'|_,'a'|_,'r'|_,'i'|_,'o'|_,'u'|_,'s'|_,' '|_,'b'|_,'a'|_,'c'|_,'k'|_,'g'|_,'r'|_,'o'|_,'u'|_,'n'|_,'d'|_
            .DB   ' '|_,'c'|_,'o'|_,'l'|_,'o'|_,'r'|_,'s'|_,'.'|_
            .DB   LF|_,CR|_,LF|_

            .DB   EOF
msg78ch:
            .DB   ' '|_,NUL,78,' '|_,EOF
msgNow:
            .DB   LF|_
            .DB   ' '|_,'N'|_,'O'|_,'W'|_,' '|_,'Y'|_,'O'|_,'U'|_,' '|_,'C'|_,'A'|_,'N'|_,' '|_,'P'|_,'E'|_,'R'|_,'F'|_,'O'|_,'R'|_,'M'|_
            .DB   ' '|_,'T'|_,'H'|_,'E'|_,' '|_,'A'|_,'U'|_,'T'|_,'O'|_,'-'|_,'A'|_,'D'|_,'J'|_,'U'|_,'S'|_,'T'|_,'M'|_,'E'|_,'N'|_,'T'|_
            .DB   ' '|_,'O'|_,'F'|_,' '|_,'T'|_,'H'|_,'E'|_,' '|_,'D'|_,'I'|_,'S'|_,'P'|_,'L'|_,'A'|_,'Y'|_,' '|_,'F'|_,'R'|_,'O'|_,'M'|_
            .DB   ' '|_,'O'|_,'S'|_,'D'|_,' '|_,'O'|_,'R'|_,' '|_,'A'|_,'U'|_,'T'|_,'O'|_,' '|_,'K'|_,'E'|_,'Y'|_,'.'|_
            .DB   LF|_,CR|_
            .DB   ' '|_,'T'|_,'h'|_,'e'|_,' '|_,'f'|_,'i'|_,'n'|_,'a'|_,'l'|_,' '|_,'r'|_,'e'|_,'s'|_,'u'|_,'l'|_,'t'|_
            .DB   ' '|_,'s'|_,'h'|_,'o'|_,'u'|_,'l'|_,'d'|_,' '|_,'b'|_,'e'|_,' '|_,'a'|_,'n'|_
            .DB   ' '|_,'a'|_,'c'|_,'c'|_,'u'|_,'r'|_,'a'|_,'t'|_,'e'|_,' '|_,'p'|_,'i'|_,'l'|_,'l'|_,'a'|_,'r'|_,'b'|_,'o'|_,'x'|_
            .DB   ' '|_,'i'|_,'m'|_,'a'|_,'g'|_,'e'|_,' '|_,'c'|_,'e'|_,'n'|_,'t'|_,'e'|_,'r'|_,'e'|_,'d'|_,' '|_,'o'|_,'n'|_
            .DB   ' '|_,'t'|_,'h'|_,'e'|_,' '|_,'s'|_,'c'|_,'r'|_,'e'|_,'e'|_,'n'|_,'.'|_
            .DB   LF|_,CR|_,LF|_

            .DB   ' '|_,'A'|_,'t'|_,' '|_,'t'|_,'h'|_,'e'|_,' '|_,'e'|_,'n'|_,'d'|_,' '|_,'o'|_,'f'|_,' '|_,'t'|_,'h'|_,'e'|_
            .DB   ' '|_,'c'|_,'a'|_,'l'|_,'i'|_,'b'|_,'r'|_,'a'|_,'t'|_,'i'|_,'o'|_,'n'|_,' '|_,'p'|_,'r'|_,'e'|_,'s'|_,'s'|_
            .DB   ' '|_,'a'|_,'n'|_,'y'|_,' '|_,'k'|_,'e'|_,'y'|_,' '|_,'t'|_,'o'|_,' '|_,'q'|_,'u'|_,'i'|_,'t'|_,' '|_
            .DB   BEL|_                   ; beep sound

            .DB   EOF
; ----------------------------------------
msgDone:                                  ; texts comply with 37 columns
            .DB   CLS|_,'D'|_,'o'|_,'n'|_,'e'|_,'!'|_,LF|_,CR|_,EOF
wrongMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'W'|_,'r'|_,'o'|_,'n'|_,'g'|_,' '|_,'v'|_,'i'|_,'d'|_,'e'|_
            .DB   'o'|_,' '|_,'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_,EOF
unsuppMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'U'|_,'n'|_,'s'|_,'u'|_,'p'|_,'p'|_,'o'|_,'r'|_,'t'|_,'e'|_
            .DB   'd'|_,' '|_,'c'|_,'o'|_,'m'|_,'m'|_,'a'|_,'n'|_,'d'|_,EOF
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
            .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;           .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
            .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   ".COM ]"
; ----------------------------------------
            .END

